@@ -60,6 +60,10 @@ public class FormStrings implements LocalizableStrings {
 
     public static final String FORM_LABEL_URL = "URL";
 
+    public static final String SOURCE_NAME_CONTACTS = "OXMF Contacts";
+
+    public static final String SOURCE_NAME_INFOSTORE = "OXMF Infostore";
+
     public FormStrings() {
         super();
     }
