@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -49,7 +49,11 @@
 
 package com.openexchange.subscribe.microformats;
 
+import java.io.BufferedReader;
+import java.io.IOException;
 import java.io.Reader;
+import java.io.StringReader;
+import java.util.ArrayList;
 import java.util.Collection;
 import java.util.LinkedList;
 import java.util.List;
@@ -59,41 +63,91 @@ import com.openexchange.subscribe.Subscription;
 import com.openexchange.subscribe.SubscriptionException;
 import com.openexchange.subscribe.SubscriptionSource;
 import com.openexchange.subscribe.microformats.datasources.OXMFDataSource;
+import com.openexchange.subscribe.microformats.parser.ObjectParser;
 import com.openexchange.subscribe.microformats.transformers.MapToObjectTransformer;
 
-
 /**
  * {@link MicroformatSubscribeService}
- *
+ * 
  * @author <a href="mailto:francisco.laguna@open-xchange.com">Francisco Laguna</a>
- *
  */
 public class MicroformatSubscribeService extends AbstractSubscribeService {
 
     private MapToObjectTransformer transformer;
+
     private OXMFParserFactoryService parserFactory;
+
     private OXMFDataSource mfSource;
+
     private List<String> containers = new LinkedList<String>();
+
     private List<String> prefixes = new LinkedList<String>();
+
     private SubscriptionSource source;
 
+    private List<ObjectParser> objectParsers = new LinkedList<ObjectParser>();
+
     public Collection getContent(Subscription subscription) throws SubscriptionException {
         Reader htmlData = mfSource.getData(subscription);
+        return getContent(htmlData);
+    }
+
+    public Collection getContent(Reader htmlData) throws SubscriptionException {
+        String data = null;
+
+        if (!objectParsers.isEmpty()) {
+            data = read(htmlData);
+            htmlData = new StringReader(data);
+        }
+
         OXMFParser parser = parserFactory.getParser();
         configureParser(parser);
         List<Map<String, String>> parsed = parser.parse(htmlData);
-        return transformer.transform(parsed);
+        List results = new ArrayList(transformer.transform(parsed));
+
+        for (ObjectParser objectParser : objectParsers) {
+            Collection parsedObjects = objectParser.parse(new StringReader(data));
+            results.addAll(parsedObjects);
+        }
+
+        return results;
+    }
+
+    public void addObjectParser(ObjectParser parser) {
+        objectParsers.add(parser);
+    }
+
+    private String read(Reader htmlData) throws SubscriptionException {
+        StringBuilder builder = new StringBuilder();
+        BufferedReader reader = null;
+        try {
+            reader = new BufferedReader(htmlData);
+            String line = null;
+            while ((line = reader.readLine()) != null) {
+                builder.append(line).append('\n');
+            }
+            return builder.toString();
+
+        } catch (IOException e) {
+            throw OXMFSubscriptionErrorMessage.IOException.create(e);
+        } finally {
+            if (reader != null) {
+                try {
+                    reader.close();
+                } catch (IOException e) {
+                }
+            }
+        }
     }
 
     public SubscriptionSource getSubscriptionSource() {
         return source;
     }
 
-    
     public void setSource(SubscriptionSource source) {
         this.source = source;
     }
-   
+
     public boolean handles(int folderModule) {
         return source.getFolderModule() == folderModule;
     }
@@ -123,11 +177,11 @@ public class MicroformatSubscribeService extends AbstractSubscribeService {
      * @param parser
      */
     public void configureParser(OXMFParser parser) {
-        for(String container : containers) {
+        for (String container : containers) {
             parser.addContainerElement(container);
         }
-        
-        for(String prefix : prefixes) {
+
+        for (String prefix : prefixes) {
             parser.addAttributePrefix(prefix);
         }
     }
@@ -145,14 +199,14 @@ public class MicroformatSubscribeService extends AbstractSubscribeService {
     public void addPrefix(String string) {
         prefixes.add(string);
     }
-    
+
     @Override
     public void modifyOutgoing(Subscription subscription) throws SubscriptionException {
         subscription.setDisplayName(getDisplayName(subscription));
     }
 
     protected String getDisplayName(Subscription subscription) {
-        return (String)subscription.getConfiguration().get("url");
+        return (String) subscription.getConfiguration().get("url");
     }
 
 }
